package com.sun.xml.internal.ws.api.server;

import com.sun.xml.internal.ws.server.provider.SyncProviderInvokerTube;
import com.sun.xml.internal.ws.server.provider.AsyncProviderInvokerTube;
import java.util.Iterator;
import java.util.logging.Level;
import com.sun.xml.internal.ws.util.ServiceFinder;
import com.sun.istack.internal.Nullable;
import com.sun.xml.internal.ws.server.provider.ProviderInvokerTube;
import com.sun.xml.internal.ws.server.provider.ProviderArgumentsBuilder;
import com.sun.istack.internal.NotNull;
import java.util.logging.Logger;

public abstract class ProviderInvokerTubeFactory<T>
{
    private static final ProviderInvokerTubeFactory DEFAULT;
    private static final Logger logger;
    
    public ProviderInvokerTubeFactory() {
    }
    
    protected abstract ProviderInvokerTube<T> doCreate(@NotNull Class<T> p0, @NotNull Invoker p1, @NotNull ProviderArgumentsBuilder<?> p2, boolean p3);
    
    public static <T> ProviderInvokerTube<T> create(@Nullable ClassLoader classLoader, @NotNull Container container, @NotNull Class<T> implType, @NotNull Invoker invoker, @NotNull ProviderArgumentsBuilder<?> argsBuilder, boolean isAsync) {
        for (ProviderInvokerTubeFactory factory : ServiceFinder.find(ProviderInvokerTubeFactory.class, classLoader, container)) {
            ProviderInvokerTube<T> tube = factory.doCreate(implType, invoker, argsBuilder, isAsync);
            if (tube != null) {
                if (ProviderInvokerTubeFactory.logger.isLoggable(Level.FINE)) {
                    ProviderInvokerTubeFactory.logger.log(Level.FINE, "{0} successfully created {1}", new Object[] { factory.getClass(), tube });
                    return tube;
                }
                return tube;
            }
            else {
                continue;
            }
        }
        return ProviderInvokerTubeFactory.DEFAULT.createDefault(implType, invoker, argsBuilder, isAsync);
    }
    
    protected ProviderInvokerTube<T> createDefault(@NotNull Class<T> implType, @NotNull Invoker invoker, @NotNull ProviderArgumentsBuilder<?> argsBuilder, boolean isAsync) {
        return isAsync ? new AsyncProviderInvokerTube(invoker, argsBuilder) : new SyncProviderInvokerTube(invoker, argsBuilder);
    }
    
    static {
        DEFAULT = new ProviderInvokerTubeFactory.DefaultProviderInvokerTubeFactory();
        logger = Logger.getLogger(ProviderInvokerTubeFactory.class.getName());
    }
    
    private static class DefaultProviderInvokerTubeFactory<T> extends ProviderInvokerTubeFactory<T>
    {
        public ProviderInvokerTube<T> doCreate(@NotNull Class<T> implType, @NotNull Invoker invoker, @NotNull ProviderArgumentsBuilder<?> argsBuilder, boolean isAsync) {
            return this.createDefault(implType, invoker, argsBuilder, isAsync);
        }
    }
}
